Define OSC 20 ' 20 MHz oscillator
'DEFINE XOUTXLAT_OFF 1 ' Uncomment to use raw coding for X10 functions

' Hardware serial port number parameters
' Set receive register to receiver enabled
DEFINE HSER_RCSTA 90h 
' Set transmit register to transmitter enabled
DEFINE HSER_TXSTA 20h 
' Set baud rate
DEFINE HSER_BAUD 2400

'Analog conversion options
DEFINE ADC_BITS 10
DEFINE ADC_CLOCK 2 'for 20 MHz oscillator
DEFINE ADC_SAMPLEUS 50 

XMIT_LED var PORTB.1
BAT_LED var PORTB.2

I var word
J var word
K var word
ADC_Data var word
ser_char var byte 'RS232 serial character
char var byte 'Character to Display
Hex_Digit var byte
Hex_Char var byte
Word_16 var word
Word_10 var word
Leading var byte
Scratch var word
Clock_Register var byte
Clock_Data var byte
House_Code var byte
Unit_Code var byte
Function_Code var byte
Serial_Status var byte
Battery_Voltage var word
Time var byte[3]
Hour var byte
Minute var byte
DOW var byte
Event_Hour var Byte
Event_Minute var byte
Event_DOW var Byte
Event_Code var Byte
Event_Command var byte
Last_Hour var byte
Last_Minute var byte
Last_DOW var byte

'PortA set for all inputs
'PortA b0-b3 are house code switches
'PortA b5 is battery voltage sensing
TRISA = %11111111

'Set PortB
TRISB = %00010000

'Set PortC
TRISC = %10010000

'Set PortD I/O Pins
'Note D0 is I2C SCL, D1 is I2C SDA
'D2 is X10 output, D3 is X10 zero-crossing input
TRISD = %01001000

'Set PortE I/O Pins
TRISE = %00000000

'Set up ADC registers
'Port A is all analog
ADCON1 = %10000000

Main:
Call Sign_On_Message
'Turn X10 Oscillator off
PortD.2 = 0
Gosub Get_House_Code
gOSUB Show_Commands

Gosub Clock_Read
Last_Minute = Minute

Gosub Flash_LEDs

Main_Loop:
'Infinite loop
Repeat

' Light LED if battery voltage is OK
' Note - PCB jumper must be in place
gosub Check_Battery

'Check for received serial data
Serial_Status = PIR1.5

If (Serial_Status = 1) then 
HSERIN[ser_char]

Select case ser_char
Case "S"
gosub Set_Time
Case "T"
Gosub Show_Time
Case "U"
Gosub Update_Event_Table
Case "E"
Gosub Show_Event_Table
Case "B"
Gosub Show_Battery_Voltage
Case "H"
Gosub Get_House_Code
Case "?"
Gosub Show_Commands
Case Else
hserout[ser_char]
end Select
ENDIF

Gosub Clock_Read
'Check to see whether we should scan for new events
if (Minute != Last_Minute) then
Last_Minute = Minute
Gosub Scan_Event_Table
Endif

Until (0) 'End of infinite loop
STOP

Sign_On_Message:
'Send CR
HSEROUT[13]
'Send characters
HSEROUT["X","1","0"," ","C","o","n","t","r","o","l","l","e","r"]
'Send CR
HSEROUT[13]
HSEROUT["V","e","r","s","."," ","1",".","0"]
'Send CR
HSEROUT[13]
HSEROUT["T","y","p","e"," ","?"," ","f","o","r"," ","c","o","m","m","a","n","d","s"]
'Send CR
HSEROUT[13]
Return

Show_Commands:
'S - Set_Time
'T - Show_Time
'U - Update_Event_Table
'E - Show_Event_Table
'B - Show_Battery_Voltage
'H - Show_House_Code
HSEROUT["S","=","S","e","t"," ","T","i","m","e", 13]
HSEROUT["T","=","S","h","o","w"," ","T","i","m","e", 13]
HSEROUT["U","=","U","p","d","a","t","e"," ","E","v","e","n","t"," ","T","a","b","l","e", 13]
HSEROUT["E","=","S","h","o","w"," ","E","v","e","n","t"," ","T","a","b","l","e", 13]
HSEROUT["B","=","S","h","o","w"," ","B","a","t","t","e","r","y"," ","V","o","l","t","a","g","e", 13]
HSEROUT["H","=","S","h","o","w"," ","H","o","u","s","e"," ","C","o","d","e", 13]
Return

Set_Time:
'send CR
HSEROUT[13]
HSEROUT["H","H",":","M","M"," ","="," "]
Hserin[ser_char]
HSEROUT[ser_char]
Time[1] = ser_char-48
Hserin[ser_char]
HSEROUT[ser_char]
ser_char = ser_char -48
Time[1] = Time[1] <<4
time[1] = time[1] | ser_char
'Discard colon character
Hserin[ser_char]
HSEROUT[ser_char]
Hserin[ser_char]
HSEROUT[ser_char]
Time[0] = ser_char-48
Hserin[ser_char]
HSEROUT[ser_char]
ser_char = ser_char -48
Time[0] = Time[0] <<4
time[0] = time[0] | ser_char

'send CR
HSEROUT[13]
HSEROUT["D","O","W","(","S","U","N","=","1",")"," ","="," "]
Hserin[ser_char]
HSEROUT[ser_char]
Time[2] = ser_char -48

'Transfer data to clock chip
'Set DOW
Clock_Register = $03
Clock_Data = Time[2]
Gosub Clock_Write
'Set Hours
Clock_Register = $02
Clock_Data = Time[1]
Gosub Clock_Write
'Set Minutes
Clock_Register = $01
Clock_Data = Time[0]
Gosub Clock_Write
'Set Seconds (effects data transfer to clock)
Clock_Register = $00
Clock_Data = $00
Gosub Clock_Write
'send CR
HSEROUT[13]

Show_Time:
'Display time
HSEROUT["T","i","m","e"," ","="," "]
Gosub Clock_Read
Word_16 = Hour
gosub Show_BCD
char = ":"
HSEROUT[char]
Word_16 = Minute
gosub Show_BCD
HSEROUT[" "]
Select Case DOW
Case 1
HSEROUT["S","U","N"]
Case 2
HSEROUT["M","O","N"]
Case 3
HSEROUT["T","U","E"]
Case 4
HSEROUT["W","E","D"]
Case 5
HSEROUT["T","H","U"]
Case 6
HSEROUT["F","R","I"]
Case 7
HSEROUT["S","A","T"]

END SELECT
'send CR
HSEROUT[13]

Return

Update_Event_Table:
'256 bytes available = 64 possible events
'Data storage format (all packed BCD, except DOW=binary)
'(H10,H1),(M10,M1),(Days of Week),(Device,Function)
'Days of week D0=Sun,D1=Mon,D2=Tue,D3=Wed,D4=Thu,D5=Fri,D6=Sat
'Get data via RS232 and Save data in EEPROM

'send CR
HSEROUT[13]
HSEROUT["S","e","n","d"," ","D","a","t","a","f","i","l","e"]
'send CR
HSEROUT[13]
For K = 0 to 63
HSERIN [WAIT("$"), hex ser_char]
Write (4*k), ser_char
HSERIN [WAIT("$"), hex ser_char]
Write (1+(4*k)), ser_char
HSERIN [WAIT("$"), hex ser_char]
Write (2+(4*k)), ser_char
HSERIN [WAIT("$"), hex ser_char]
Write (3+(4*k)), ser_char
'Show progress
IF ((K//4) == 0) then
HSEROUT["*"]
Endif

Next K
'send CR
HSEROUT[13]
HSEROUT["D","o","n","e"]
'send CR
HSEROUT[13]

Return

Show_Event_Table:
For I = 0 to 255
Read I, Word_16
Hserout["$"]
Gosub Show_BCD
Hserout[" "]
IF (((I+1)//4) == 0) then
'send CR
HSEROUT[13]
Endif
Next
'send CR
HSEROUT[13]
Return

Scan_Event_Table:
'Convert DOW to equivalent bit position (Sun=b0, etc.)
DOW = %00000001 << (DOW-1)
For I = 0 to 63
Read ((4*I)+1), Event_Minute
If (Event_Minute == Minute) then
Read (4*I), Event_Hour
If (Event_Hour == Hour) then
Read ((4*I)+2), Event_DOW
If ((Event_DOW & DOW)!=0) then
'Event time matches current time if we are here
'Extract device and command codes from stored byte
Read ((4*I)+3), Event_Code
Unit_Code = Event_Code & %11110000
Unit_Code = Unit_Code >> 4
Event_Command =  Event_Code & %00001111
Word_16 = Hour
gosub Show_BCD
char = ":"
HSEROUT[char]
Word_16 = Minute
gosub Show_BCD
HSEROUT[" "]
Select Case DOW
Case 1
HSEROUT["S","U","N"]
Case 2
HSEROUT["M","O","N"]
Case 4
HSEROUT["T","U","E"]
Case 8
HSEROUT["W","E","D"]
Case 16
HSEROUT["T","H","U"]
Case 32
HSEROUT["F","R","I"]
Case 64
HSEROUT["S","A","T"]
END SELECT
hserout[" ","X","1","0","="]
Word_16 = Unit_Code
gosub Show_BCD
hserout[" "]
Word_16 = Event_Command
gosub Show_BCD
'send CR
HSEROUT[13]

'Command Codes for this controller
'0 - All_units_off
'1 - All_lights_on
'2 - All_lights_off
'3 - On
'4 - Off
'5 - Bright_10 (Brighten 10%)
'6 - Bright_20 (Brighten 20%)
'7 - Bright_40 (Brighten 40%)
'8 - Bright_80 (Brighten 80%)
'9 - Dim_10 (Dim 10%)
'A - Dim_20 (Dim 20%)
'B - Dim_40 (Dim 40%)
'C - Dim_80 (Dim 80%)

'X10 Function Codes used by XOUT Command
' %10010 Turn module on
' %11010 Turn module off
' %11100 Turn all modules off
' %10100 Turn all light modules on
' %10000 Turn all light modules off
' %10110 Brighten light module
' %11110 Dim light module

Select Case Event_Command
Case 0
'All_units_off
Function_Code = %11100
Gosub Send_X10_Command
Case 1
'All_lights_on
Function_Code = %10100
Gosub Send_X10_Command
Case 2
'All_lights_off
Function_Code = %10000
Gosub Send_X10_Command
Case 3
'On
Function_Code = %10010
Gosub Send_X10_Unit_Code
Gosub Send_X10_Command
Case 4
'Off
Function_Code = %11010
Gosub Send_X10_Unit_Code
Gosub Send_X10_Command
Case 5
'Bright_10 (Brighten 10%)
Function_Code = %10110
Gosub Send_X10_Unit_Code
Gosub Send_X10_Command
Pause 250
Case 6
'Bright_20 (Brighten 20%)
Function_Code = %10110
Gosub Send_X10_Unit_Code
For J = 1 to 2
Gosub Send_X10_Command
Pause 250
Next J
Case 7
'Bright_40 (Brighten 40%)
Function_Code = %10110
Gosub Send_X10_Unit_Code
For J = 1 to 4
Gosub Send_X10_Command
Pause 250
Next J
Case 8
'Bright_80 (Brighten 80%)
Function_Code = %10110
Gosub Send_X10_Unit_Code
For J = 1 to 8
Gosub Send_X10_Command
Pause 250
Next J
Case 9
'Dim_10 (Dim 10%)
Function_Code = %11110
Gosub Send_X10_Unit_Code
Gosub Send_X10_Command
Pause 250
Case 10
'Dim_20 (Dim 20%)
Function_Code = %11110
Gosub Send_X10_Unit_Code
For J = 1 to 2
Gosub Send_X10_Command
Pause 250
Next J
Case 11
'Dim_40 (Dim 40%)
Function_Code = %11110
Gosub Send_X10_Unit_Code
For J = 1 to 4
Gosub Send_X10_Command
Pause 250
Next J
Case 12
'Dim_80 (Dim 80%)
Function_Code = %11110
Gosub Send_X10_Unit_Code
For J = 1 to 8
Gosub Send_X10_Command
Pause 250
Next J
End Select

Endif
Endif 
Endif
Next I

Return


Send_X10_Unit_Code:
'House_Code and Unit_Code set before entry
'House_Code = 0 'Code for A
'Unit_code = 15 'Code for 15
XMIT_LED = 1
XOUT PORTD.2,PORTD.3,[House_Code\Unit_Code]
XMIT_LED = 0
Pause 250

Return

Send_X10_Command:
'House_Code and Function_Code set before entry
'House_Code = 0 'Code for A
'Unit_code = 15 'Code for 15

' Function Codes used by XOUT Command
' %10010 Turn module on
' %11010 Turn module off
' %11100 Turn all modules off
' %10100 Turn all light modules on
' %10000 Turn all light modules off
' %10110 Brighten light module
' %11110 Dim light module
XMIT_LED = 1
XOUT PORTD.2,PORTD.3,[House_Code\Function_Code]
XMIT_LED = 0
Pause 250
Return

Flash_LEDs:
'Flash LEDs to verify operation when power is first applied
BAT_LED = 1
XMIT_LED = 0
pause 250
For I = 0 to 7
Toggle BAT_LED
Toggle XMIT_LED
Pause 250
Next I
BAT_LED = 0
XMIT_LED = 0
Return

Clock_Write:
'Note Port D0 is I2C SCL, Port D1 is I2C SDA
'Example Code
'I2CWRITE PORTD.1,PORTD.0,%11010000,$Address,[Clock_Data]
I2CWRITE PORTD.1,PORTD.0,%11010000,Clock_Register,[Clock_Data]
RETURN

Clock_Read:
'Note Port D0 is I2C SCL, Port D1 is I2C SDA
I2CREAD PORTD.1,PORTD.0,%11010001,$03,[DOW]
I2CREAD PORTD.1,PORTD.0,%11010001,$02,[Hour]
I2CREAD PORTD.1,PORTD.0,%11010001,$01,[Minute]
'Check for valid data
Clock_Data = Hour
I2CREAD PORTD.1,PORTD.0,%11010001,$02,[Hour]
If (Hour != Clock_Data) then
Goto Clock_Read
Endif
RETURN

Get_House_Code:
House_Code = 0
For I = 0 to 3
House_Code = House_Code >>1
'Analog inputs on Port A
ADCIN I, ADC_Data
If ADC_Data >511 then
House_Code = House_Code + 8
ENDIF
Next I
Gosub Show_House_Code
Return

Show_House_Code:
'Update values for possible jumper change
'Note - "A" is decimal 65 in ASCII
HSEROUT["H","o","u","s","e"," ","C","o","d","e"," ","="," "]
Hserout[House_Code + 65]
'send CR
HSEROUT[13]
Return

Check_Battery:
ADCIN 4,ADC_Data
If (ADC_Data > 425) then
BAT_LED = 1
Else
BAT_LED = 0
Endif
Return

Show_Battery_Voltage:
ADCIN 4,ADC_Data
'Calibrated values  5.00 volts = 1023
Battery_Voltage = (31*ADC_Data)/64
Word_10 = Battery_Voltage
HSEROUT["B","a","t","t","e","r","y"," ","="," "]
' Get 100 digit
Scratch = Word_10/100
Word_10 = Word_10//100
char = 48 + Scratch
HSEROUT[char]
HSEROUT["."]
' Get 10 digit
Scratch = Word_10/10
Word_10 = Word_10//10
char = 48 + Scratch
HSEROUT[char]
' Show units digit
char = 48 + Word_10
HSEROUT[char]
'send CR
HSEROUT[13]
Return

Show_16:
' Displays word as a base-16 number
' Display packed BCD numbers, also
' Word_16 set before entry
' Get $F000 digit
Hex_Digit = Word_16 >> 12
Gosub Show_Hex
' Get $0F00 digit
Scratch = $0F00 & Word_16
Hex_Digit = Scratch >> 8
Gosub Show_Hex
' Get $00F0 digit
Scratch = $00F0 & Word_16
Hex_Digit = Scratch >> 4
Gosub Show_Hex
' Get $000F digit
Hex_Digit = Word_16 & $000F
Gosub Show_Hex
'send CR
HSEROUT[13]
Return

Show_BCD:
' Displays Byte as a base-16 number
' Displays BCD numbers, also
' Word_16 set before entry
' Get $00F0 digit
Scratch = $00F0 & Word_16
Hex_Digit = Scratch >> 4
Gosub Show_Hex
' Get $000F digit
Hex_Digit = Word_16 & $000F
Gosub Show_Hex
Return


Show_Hex:
Select Case Hex_Digit
    Case IS < 10
    Hex_Char = Hex_Digit + 48
    Case IS > 9
    Hex_Char = Hex_Digit + 55
End Select
'Send character
HSEROUT[Hex_Char]
Return


Show_10:
' Displays word as a base-10 number
' Word_10 set before entry
Leading = 0  ' Leading zero flag
' Get 10,000 digit
Scratch = Word_10/10000
Word_10 = Word_10//10000
If (scratch == 0) then
Leading = 1
char = " "
else
Leading = 0
char = 48 + Scratch
Endif
HSEROUT[char]
' Get 1,000 digit
Scratch = Word_10/1000
Word_10 = Word_10//1000
If ((scratch == 0) AND (Leading == 1)) then
Leading = 1
char = " "
else
Leading = 0
char = 48 + Scratch
Endif
HSEROUT[char]
' Get 100 digit
Scratch = Word_10/100
Word_10 = Word_10//100
If ((scratch == 0) AND (Leading == 1)) then
Leading = 1
char = " "
else
Leading = 0
char = 48 + Scratch
Endif
HSEROUT[char]
' Get 10 digit
Scratch = Word_10/10
Word_10 = Word_10//10
If ((scratch == 0) AND (Leading == 1)) then
Leading = 1
char = " "
else
Leading = 0
char = 48 + Scratch
Endif
HSEROUT[char]
' Show units digit
char = 48 + Word_10
HSEROUT[char]
'send CR
HSEROUT[13]
Return

End
